
#Input stream data

#reading stream data is time consuming
#Therefore, placed in separate cell in order to load values from excel only once
#OBS! "using xxxxx" cannot be placed inside functions, hence call functions globally

using JuMP       #use of package JuMP to formulate optimization problem
using CPLEX      #use of package CPLEX to solve the LP problem
using Distances  #use package to calculate hamming distances

#[IMPORTANT] 
#the algorithm is designed so that pressure changing streams must be listed first
#do not distinguish between hot and cold, can be found by comparing TS and TT 
#the mapping is done such that the stream on location x in vector TS..
#..has corresponding values at location x in TT, mCp etc. 

using ExcelReaders

f = openxl("Input_streamdata.xlsx")

#remember to update cell G2
nPr=readxl(f, "Streamdata!G2:G2"); nPr=Int(nPr[1]);         #no. of dynamic streams
nSt=readxl(f, "Streamdata!I2:I2"); nSt=Int(nSt[1]);         #no. of static streams

#remember to update cellnumbers
TS=readxl(f, "Streamdata!C5:C11");                          #supply temp 
TT=readxl(f, "Streamdata!D5:D11");                          #target temp 
Pr=readxl(f, "Streamdata!G5:G8");                           #pressure ratio
mCp=readxl(f, "Streamdata!H5:H11");                         #flow rate capacity 

#remember to update values for below ambient cases
DTmin=readxl(f, "Streamdata!A2:A2"); DTmin=Int(DTmin[1]);   #delta T min
Tamb=readxl(f, "Streamdata!B2:B2");  Tamb=Float64(Tamb[1]); #ambient temp
Thu=readxl(f, "Streamdata!C2:C2");   Thu=Float64(Thu[1]);   #hot utility temp
Tcu=readxl(f, "Streamdata!D2:D2");   Tcu=Float64(Tcu[1]);   #cold utility temp
κ=readxl(f, "Streamdata!E2:E2");     κ=Float64(κ[1]);       #kappa
η=readxl(f, "Streamdata!F2:F2");     η=Float64(η[1]);       #polytropic efficiency 
n=readxl(f, "Streamdata!H2:H2");     n=Int(n[1]);           #number of stream splits
T=273.15;                                                   #kelvin temperature





#mapping of genes from genotype to phenotype

function genMapping(genPop, paraNum, ub, lb)

popSize = length(genPop);                #population size
phenPop = zeros(popSize, paraNum);       #phenotype representation of population
stepSize=zeros(Float64, paraNum);        #discretization of variable range
    
for i=1:paraNum
    stepSize[i] = (ub-lb)/((2^nBits)-1); #evenly distr density of variable range
end 

for i = 1:popSize
    k = 1;
    for j = 1:paraNum
        phenPop[i,j] = (parse(Int64, genPop[i][k:k+nBits-1], 2))*stepSize[j]+lb   
        k = k+nBits
    end 
end 
    
return phenPop                           #return a population of phenotype values
end

#fitness function/inner loop

function computeFitness(phenPop)

#output values to comparison of solutions
    
exergy = zeros(length(phenPop[:,1]));       #fitness value of individuals in pop p
Uhot = zeros(length(phenPop[:,1]));         #hot utility of indiv solutions in pop p
Ucold = zeros(length(phenPop[:,1]));        #cold utility of indiv solutions in pop p    
mfr = zeros(nPr, n, length(phenPop[:,1]));  #mCp of each stream split in pop p
TIN = zeros(nPr, n, length(phenPop[:,1]));  #input temp to pressure changing units
TOUT = zeros(nPr, n, length(phenPop[:,1])); #output temp to pressure changing units
Work = zeros(nPr, n, length(phenPop[:,1])); #work from each pressure changing unit


    #for each individual solution in population p we calculate the fitness value
    for i=1:length(phenPop[:,1])                                
        
        #extract info from each gene and calculate pressure ratio and output temp
        Tin = collect(reshape(phenPop[i,:], nPr, n)); 
        
        PR = []; Tout = zeros(nPr, n);
       
        PR = collect((Pr[s])^((κ-1)/κ) for s=1:length(Pr));                       
        
        for s = 1:nPr
            for b = 1:n
                Tout[s,b] = ((Tin[s,b]+273.15)/(PR[s]))-273.15; 
            end 
        end 
        
        #[split operator] 
        #generate new streams; one segment before unit, one after unit
        #[OBS!] pressure changing streams must be listed first in stream data
                
        TSnew = zeros(2*nPr*n); 
        TTnew = zeros(2*nPr*n); 
        mCpnew = zeros(2*nPr*n); 
        W = zeros(nPr, n);
                
        k = 1;
        for s = 1:nPr
            for b = 1:n
                TSnew[k] = TS[s];             #supply temp of segment before unit 
                TSnew[k+1] = Tout[s,b];       #supply temp of segment after unit
                TTnew[k] = Tin[s,b];          #target temp of segment before unit    
                TTnew[k+1] = TT[s];           #target temp of segment after unit
                mCpnew[k] = mCp[s];           #mCp of segment before unit
                mCpnew[k+1] = mCp[s];         #mCp of segment after unit
                k = k+2; 
                    
                W[s,b] = (Tout[s,b]-Tin[s,b])*mCp[s]; #work from each unit
            end 
        end 
       
        #for later simplicity we add the static streams to the new vectors
                
        append!(TSnew, TS[nPr+1:end]);        
        append!(TTnew, TT[nPr+1:end]);        
        append!(mCpnew, mCp[nPr+1:end]);      
                                 
        #generate the heat cascade: 
        #1. the cold temperature side 
        #2. the hot temperature side 
        #3. number of temperature intervals
                
        Tcold = []; Thot = []; K = [];        
        for j = 1:length(TSnew)               #create heat casc from all supply temp
            if TSnew[j] >= TTnew[j]           #hot stream
               push!(Tcold, TSnew[j]-DTmin);  #adjust for delta Tmin
            else push!(Tcold, TSnew[j])       #cold stream
            end
        end
        
        #to create the heat cascade we need to add the max/min temps in the system
        push!(Tcold, min(Tamb,Tcu));                                   
                                    
        Tcold = reverse(sort(union(Tcold)));  #sort the cold temp side
        Thot = Tcold+DTmin;                   #create hot side of heat casc 
        K = length(Tcold);                    #no of temp intervals in heat casc
         
        #for testing/debugging of function:        
        #println("Tin:  ", Tin)
        #println("Tout:  ", Tout)
        #println("TSnew:  ", TSnew)
        #println("TTnew:  ", TTnew)
        #println("mCpnew:  ", mCpnew)
        #println("Work:  ", W)
        #println("Tcold:  ", Tcold)
        #println("Thot:  ", Thot)
        #println("K:  ", K)
        
         #for simplicity, the heat in each interval from the static streams..
        #..and the pressure changing streams are calculated separately.
        
        #STATAIC STREAMS
        #heat from hot and cold streams to interval k in K
                
        QS = zeros(nSt, K); q = 1;
                            
for s = nPr+1:length(TS)
    if TS[s] > TT[s]
                    
    for k = 1:K-1  
    if (TT[s]<=Thot[k+1]||(TT[s]<=Thot[k]&&TT[s]>=Thot[k+1]))&&TS[s]>=Thot[k]
    QS[q,k] = mCp[s]*(Thot[k]- max(Thot[k+1], TT[s]))
    end
    end
                
                    
    elseif TS[s] < TT[s]
                  
    for k = 1:K-1
    if TS[s]<=Tcold[k+1] && ( TT[s]>=Tcold[k] || ( TT[s]<=Tcold[k] && TT[s]>=Tcold[k+1] ) )
        QS[q,k] = mCp[s]*(Tcold[k+1] - min(Tcold[k], TT[s]))
    end
    end
                   
    end
    q = q+1;
end   
               
        #for simplicity, sum of heat in each interval for the static streams
                
        Qksum = zeros(K)
        for k = 1:K
            Qksum[k] = sum(QS[1:end,k])
        end
        
        #PRESSURE CHANGING STREAMS
        #heat from pressure changing streams depends on the variable mass fraction
        #we calculate the heat supply/demand beforehand - 
        #and multiply with a fraction variable 0<=f<=1 in the optimization model
                
        #heat from hot and cold streams to interval k in K
                
        QP = zeros(2*nPr*n, K); p = 1;
                            
for s = 1:2*nPr*n
            
if TSnew[s] > TTnew[s]
    
for k = 1:K-1  
if (TTnew[s]<=Thot[k+1]||(TTnew[s]<=Thot[k]&&TTnew[s]>=Thot[k+1]))&&TSnew[s]>=Thot[k]
    QP[p,k] = mCpnew[s]*(Thot[k]- max(Thot[k+1], TTnew[s]));
end
end
                
            
elseif TSnew[s] < TTnew[s]
                 
for k = 1:K-1
if TSnew[s]<=Tcold[k+1]&&(TTnew[s]>=Tcold[k]||
            (TTnew[s]<=Tcold[k]&&TTnew[s]>=Tcold[k+1]))
    QP[p,k] = mCpnew[s]*(Tcold[k+1] - min(Tcold[k], TTnew[s]));
end
end
                   
end
p = p+1;
end 
        
        #heat content of pressure changing stream segments are calculated above..
        #..and listed in a vector QP[p,k].  
        #p is the no. of stream segments and k is the no. of temp intervals. 
        #Segments belonging to the same branch has the same flow fraction, f,..
        #..hence the array must be reshaped to the form QP[z,n,nPr,k]. 
        #For each interval k,..
        #..sum for all pressure changing streams and for all branches of these streams.
        #ex. sum(QP[z,1,1,1]*f[1,1] for z = {1,2}, s = 1, b = 1, k = 1)
                
        #reshape QP so to be compatible with f[s,b]        
        Z = 2;
        QPr = reshape(QP, Z, n, nPr, K);                            
             
        #LP-model                                            
        HeatCasc=Model(solver=CplexSolver())
        @variable(HeatCasc, r[1:K] >= 0);
        @variable(HeatCasc, Qhu >= 0);
        @variable(HeatCasc, 0 <= f[s = 1:nPr, b = 1:n] <= 1);
                                                                  
        @objective(HeatCasc, Min, 
                (1-(Tamb+T)/(Thu+T))*Qhu + ((Tamb+T)/(Tcu+T)-1)*r[K-1] + 
                sum(f[s,b]*W[s,b] for s=1:nPr, b=1:n));
                             

        @constraint(HeatCasc, FirstInt[k = 1], 
                    r[k] - Qhu == 
                    sum(QPr[z,b,s,k]*f[s,b] for s=1:nPr, b=1:n, z=1:Z) + Qksum[k] );
                                        
        @constraint(HeatCasc, ResInt[k = 2:K],
                    r[k] - r[k-1] == 
                    sum(QPr[z,b,s,k]*f[s,b] for s=1:nPr, b=1:n, z=1:Z) + Qksum[k] );
                                                    
        @constraint(HeatCasc, frac[s = 1:nPr], sum(f[s,b] for b = 1:n) == 1);
                                                                
        
        #to avoid print put from the optimization                                                        
        WW = STDOUT;                                        
        redirect_stdout();
        solve(HeatCasc);                                                                        
        redirect_stdout(WW);                                       
       
        exergy[i] = getobjectivevalue(HeatCasc);     
        Uhot[i] = getvalue(Qhu); 
        Ucold[i] = getvalue(r[K-1]);
        for s = 1:nPr
            for  b = 1:n
                 mfr[s,b,i] = getvalue(f[s,b]);      
                 Work[s, b, i] = W[s, b];            
                 TIN[s, b, i] = Tin[s, b];           
                 TOUT[s, b, i] = Tout[s, b];                                                   
            end
        end
        
        #for testing/debugging of function:                                                        
        #println("exergy:   ",exergy)
        #println("Qhu:    ", Uhot)
        #println("mfr:    ", mfr)
        #println("Work:  ", Work)
        #println("Tin:   ", TIN)
        #println("Tout:   ", TOUT)
                                                            
    end
                                                            
        indEx = indmin(exergy);
        bestfit_Uhot = Uhot[indEx];
        bestfit_work = collect(Work[s,b,i] for s = 1:nPr, b = 1:n, i = indEx);
        bestfit_TIN = collect(TIN[s,b,i] for s = 1:nPr, b = 1:n, i = indEx);  
        bestfit_TOUT = collect(TOUT[s,b,i] for s = 1:nPr, b = 1:n, i = indEx);                                                                          
        bestfit_mfr = collect(mfr[s,b,i] for s = 1:nPr, b = 1:n, i = indEx); 
                                                                                                                        
    return exergy, Uhot, Work, TIN, TOUT, mfr; #return solution and fitness measure
end

#elitist selection
#the very best individuals directly survive to the next generation 

function elitistSel(fitness, ε)
    
    popSize = length(fitness[:,1]);    #population size
    M = maximum(fitness) + 100;        #avoid choosing the same individual
    nε = Int(round(popSize*ε));        #number of "free survivors" 
    fitVal = collect(fitness);                  
    
    E = zeros(Int, nε)
    for i=1:nε
        bestfit=indmin(fitVal);        #returns a tuple of (val,indx)
        E[i] = bestfit;                #save the position of the best individual
        fitVal[bestfit] = M;           #remove best value to find second best one
    end
    return E
end

#how easily find the index of the n maximum values of a vector?!
#should be a function for this in julia, but cannot find any

#pairing of parent individuals

function crowdingPairing(genPop)

pairSize = Int(length(genPop)/2);            
selVec = collect(1:length(genPop));          

#pairing of parents
parents = Array{Tuple}(pairSize);               
for i = 1:pairSize
    
    indxA = rand(1:length(selVec));
    parentA = selVec[indxA];
    deleteat!(selVec, indxA);
    
    indxB = rand(1:length(selVec));
    parentB = selVec[indxB];
    deleteat!(selVec, indxB);
    
   parents[i] = (parentA, parentB); 
    
end 
    
return parents  #return population of paired parent individuals
end


#uniform crossover

function crowdingCrossover(parents, genPop, α)

pairSize = length(parents);                 #number of pairs
crossVec = zeros(Int, pairSize);

for i = 1:pairSize
    if rand() <= α
        crossVec[i] = 1;
    end
end 

offsprings = Array{Tuple}(sum(crossVec)); 

j = 1;
for i = 1:pairSize
    
    if crossVec[i] == 1                     #crossover probability
          
       parentA = genPop[parents[i][1]];     #random chosen individuals       
       parentB = genPop[parents[i][2]];     #random chosen individuals
            
       mask = rand(0:1, length(genPop[1])); #uniform crossover mask
            
       newbornA = zeros(Int, length(mask)); #generate new individuals
       newbornB = zeros(Int, length(mask)); #generate new individuals
            
       for a = 1:length(mask)
            
           if mask[a] == 1
      
              newbornA[a] = parse(Int, parentA[a]);
              newbornB[a] = parse(Int, parentB[a]);
                    
           elseif mask[a] == 0
                
                  newbornA[a] = parse(Int, parentB[a]);
                  newbornB[a] = parse(Int, parentA[a]);
           end
       end
            
       childA = join(newbornA);
       childB = join(newbornB);
        
       #create new individuals from parents    
       offsprings[j] = (childA, childB);  
       j = j+1;
      end     
end

par = Array{Tuple}(length(offsprings));

j = 1;
for i = 1:length(crossVec)
    if crossVec[i] == 1
        par[j] = parents[i];
        j = j+1;
    end
end
    
return offsprings, par   #return tuple of offsprings
end 



#mutation of offsprings

function crowdingMut(offsprings, μ)
    
pairSize = length(offsprings);         #number of pairs
chromSize = length(offsprings[1][1]);  #size of chromosomes
    
for i = 1:pairSize
    
    newGenes = zeros(Int, 2, chromSize);
    for j = 1:2
        if rand() <= μ                 #mutation probability

           mutChrom = rand(0:1, chromSize);    
  
           for k = 1:chromSize
            
               if mutChrom[k] == 1
                   
                  if parse(Int, offsprings[i][j][k]) == 0;
                     newGenes[j,k] = 1;
                        
                   elseif  parse(Int, offsprings[i][j][k]) == 1
                           newGenes[j,k] = 0;
                   end
                    
                else newGenes[j,k] = parse(Int, offsprings[i][j][k]);
                     
                end     
            end
            
        
        else 
             for k = 1:chromSize
                 newGenes[j,k] = parse(Int, offsprings[i][j][k]);
             end 
             
        end 
    end
    offsprings[i] = (join(newGenes[1,:]), join(newGenes[2,:]))
end
    
return offsprings #return mutated population
end

#record hamming distance between parent genes and offspring genes

function DISTANCE(par, genPop, offsprings)
    
distArr = zeros(Int, 2, 2, length(offsprings));

for i = 1:length(offsprings)
    for m = 1:2
        for n = 1:2
            parent = parse.(split(genPop[par[i][m]],""));
            child = parse.(split(offsprings[i][n],""));
            distArr[m,n,i] = evaluate(Hamming(), parent, child);
        end
    end
end
    
return distArr #return array of hamming distances
end

#match each offspring with the most similar parent

function MATCH(distArr, offsprings, par)

match = Array{Tuple}(length(offsprings));
for i = 1:length(offsprings)
    d1 = distArr[1,1,i] + distArr[2,2,i];
    d2 = distArr[1,2,i] + distArr[2,1,i];
    if d1 <= d2
         match[i] = (par[i][1], offsprings[i][1]), (par[i][2], offsprings[i][2])
    else match[i] = (par[i][1], offsprings[i][2]), (par[i][2], offsprings[i][1])
    end
end

genOff = Array{String}(length(offsprings)*2);
k = 1;
for i = 1:length(match)
    for j = 1:2
        genOff[k] = match[i][j][2]
        k = k+1
    end 
end

return match, genOff   #return the matches
end




#replacement; generalized replacement rule

function replacement(genPop, phenPop, fitness, genOff, phenOff, fitOff, match, φ, elit)

k = 0;
for i = 1:length(match)
    for j = 1:2
        
        k = k+1

        indxP = match[i][j][1];
        Pf = fitness[1][indxP];
        Cf = fitOff[1][k];
   
        if Cf < Pf
            Pc = Cf/(Cf + φ*Pf);
        elseif Cf == Pf
            Pc = 0.5;
        elseif Cf > Pf
            Pc = (φ*Cf)/(φ*Cf + Pf);
        end 

        indx = indmax(fitness[1]);
        N = 0;
        for n = 1:length(elit)
            if indxP == elit[n]
               N = 1;
            end 
        end 
                
        if rand() <= Pc && (N != 1 || (N == 1 && indx == indxP))
        
           genPop[indxP] = genOff[k]; 
           phenPop[indxP,:] = phenOff[k,:];
           fitness[1][indxP] = fitOff[1][k]; 
           fitness[2][indxP] = fitOff[2][k]; 
           for f = 3:6
               fitness[f][:,:,indxP] = fitOff[f][:,:,k];
           end         
                
        elseif N == 1 && indx != indxP
               genPop[indx] = genPop[indxP]; 
               phenPop[indx,:] = phenPop[indxP,:];
               fitness[1][indx] = fitness[1][indxP]; 
               fitness[2][indx] = fitness[2][indxP]; 
               for f = 3:6
                   fitness[f][:,:,indx] = fitness[f][:,:,indxP];
               end  
                      
               if rand() <= Pc
                  genPop[indxP] = genOff[k]; 
                  phenPop[indxP,:] = phenOff[k,:];
                  fitness[1][indxP] = fitOff[1][k]; 
                  fitness[2][indxP] = fitOff[2][k]; 
                  for f = 3:6
                      fitness[f][:,:,indxP] = fitOff[f][:,:,k];
                  end   
               end
        end    
            
    end
end         
                
 
return genPop, phenPop, fitness
end 

#GA with crowding
#MAIN FILE

tic() #start timing

# ------------initialization of the GA-----------------------------------------
paraNum = nPr*n;       #no. of decision variables featuring one solution
lb = Tamb;             #lower bound on range for decision variables in paraNum
ub = Thu;              #upper bound on range for decision variables in paraNum
nBits = 11;            #number of bits needed to represent the variable range 
                       #in binary numbers
ε = 0.01;              #rate of elitism
φ = 0.15;              #scaling factor
μ = 0.10;              #mutation rate 1/(nBits*nPr*n)
α = 0.75;              #crossover probability
popSize = 20*paraNum;  #size of a population must be an even number
genCount = 300;      #generation counter
#-----------------------------------------------------------------------------

#initial population in chromosome representation
genPop = Array{String}(popSize)
seed = 123;
for i = 1:popSize
    srand(seed);
    genPop[i] = join(rand(0:1, nBits*nPr*n));
    seed = seed + i;
end

#initial mapping to phenotype representation
phenPop = genMapping(genPop, paraNum, ub, lb);

#initial fitness evaluation
fitness = computeFitness(phenPop);

#------------------------------start outer loop----------------------------------
bestfit = [];
averagefit = [];
for m = 1:genCount
    
    #print of found optimum
    push!(bestfit, minimum(fitness[1]));
    push!(averagefit, sum(fitness[1])/length(fitness[1]));
    
    #elitism
    elit = elitistSel(fitness[1], ε);
    
    #----------------crowding-------------------------------
    #random pairing of parents
    parents = crowdingPairing(genPop);
        
    #crossover: uniform crossover
    outCross = crowdingCrossover(parents, genPop, α);
    offsprings = outCross[1];
    par = outCross[2];                                      
    
    #mutation: Flipping based on mutation mask
    mutOff = crowdingMut(offsprings, μ);
        
    #calculate distance
    distArr = DISTANCE(par, genPop, offsprings);
      
    #matching of most similar parent/child 
    matching = MATCH(distArr, offsprings, par);
    match = matching[1];
    genOff = matching[2];
                                                           
    #assign fitness to offsprings
    phenOff = genMapping(genOff, paraNum, ub, lb);
    
    #--------------------------inner loop------------------------------------
    fitOff = computeFitness(phenOff);
    #------------------------------------------------------------------------ 
    
    
    #---------Replacement----------------------------------------------------
    #local tournament
    outRep = replacement(genPop,phenPop,fitness,genOff,phenOff,fitOff,match,φ,elit);
    genPop = outRep[1];
    phenPop = outRep[2];
    fitness = outRep[3];
    #------------------------------------------------------------------------ 
    
    #for testing/debugging of function:
    #println("parents:      ",parents)
    #println("offsprings:   ",offsprings)
    #println("par:          ",par)
    #println("mutOff:       ",mutOff)
    #println("distArr:      ",distArr)
    #println("match:        ",match)
    #println("genOff:       ",genOff)
    #println("phenOff:      ",phenOff)
    #println("fitOff:       ",fitOff)
    #println("iteration:  ", m)
    
    #print of found optimum
    #push!(bestfit, minimum(fitness[1])); 
    #push!(averagefit, sum(fitness[1]/length(fitness[1])))
    
    if m == genCount
        
    #for testing/debugging of function: 
    #rintln("exergy:  ", fitness[1])
    #rintln("mass flow fraction:  ", fitness[6])
    #rintln("hot utility:  ", fitness[2])
    #rintln("work:  ", fitness[3])
    #rintln("Tin:  ", fitness[4])
    #rintln("Tout:  ", fitness[5])
        
        
    bf = minimum(fitness[1]);
    indVec = [];
    for i = 1:length(fitness[1])
        if fitness[1][i] ==bf;
           push!(indVec, i);
        end                                                                                            
    end
        
    numSol = length(indVec);
    println("                                                                    ")                                                                                            
    println("Results from GA CROWDING last iteration m = ",m)
    println("                                                                    ")                                                                                                    
    println("number of optimal solutions found:  ",numSol)                                                                                           
    println("____________________________________________________________________")
    println("                                                                    ")                                                                                             
    for i = 1:length(indVec)                                                                                                                                                                                             
        println("Solution $i:  ") 
        println("                                                                ")                                                                                                                                                                                                       
        println("Exergy value:                   ", fitness[1][indVec[i]])                                                                                                    
        println("Hot utility:                    ", fitness[2][indVec[i]]) 
        println("Work:                           ", fitness[3][:,:,indVec[i]])                                                                                                    
        println("Fraction of mass flow rate:     ", fitness[6][:,:,indVec[i]])                                                                                                    
        println("Input temperature to unit:      ", fitness[4][:,:,indVec[i]])        
        println("Output temperature from unit:   ", fitness[5][:,:,indVec[i]])                                                                                                    
        println("________________________________________________________________")                                                                                                    
        println("                                                                ")                                                                                                    
        println("                                                                ")     
    end 
 end
      
end

#------------------------------end outer loop------------------------------------ 
toc() #end timing


#printing of population evolution

#println("bestfit:  ",bestfit)
#println("averagefit:  ",averagefit)
#using Gadfly   #for plotting
#plot(x=1:genCount+1, y=bestfit, Geom.line)
#plot(x=1:genCount+1, y=averagefit, Geom.line)

#using DataFrames
#xs=1:genCount+1;
#df_bestfit = DataFrame( x=xs, y=bestfit, Properties="Highest fitness in population x")
#df_averagefit = DataFrame(x=xs, y=averagefit, Properties="Average fitness in population x")
#df=vcat(df_bestfit, df_averagefit)
#p = plot(df, x=:x, y=:y, color=:Properties, Geom.line, 
#      Guide.title("Population fitness"),
#      Guide.xlabel("Population iterations"), Guide.ylabel("fitness values") )

#draw(SVGJS(20cm, 12cm), p)


#plot average population fitness and best fitness value throughout the search

using Gadfly   #for plotting
using DataFrames

xs=1:genCount;
df_bestfit = DataFrame( x=xs, y=bestfit, 
    Properties="Highest fitness in population x")
df_averagefit = DataFrame(x=xs, y=averagefit, 
    Properties="Average fitness in population x")
df=vcat(df_bestfit, df_averagefit)
p = plot(df, x=:x, y=:y, color=:Properties, Geom.line, 
      Guide.title("Population fitness"),
      Guide.xlabel("Population iterations"), Guide.ylabel("Fitness values") )

draw(SVGJS(25cm, 16cm), p)
    


    
    


