
#Input stream data

#reading stream data is time consuming
#Therefore, placed in separate cell in order to load values from excel only once
#OBS! "using xxxxx" cannot be placed inside functions, hence call functions globally

using JuMP    #use of package JuMP to formulate optimization problem
using CPLEX   #use of package CPLEX to solve the LP problem

#[IMPORTANT] 
#the algorithm is designed so that pressure changing streams must be listed first
#do not distinguish between hot and cold, can be found by comparing TS and TT 
#the mapping is done such that the stream on location x in vector TS..
#..has corresponding values at location x in TT, mCp etc. 

using ExcelReaders

f = openxl("Input_streamdata.xlsx")

#remember to update cell G2
nPr=readxl(f, "Streamdata!G2:G2"); nPr=Int(nPr[1]);         #no. of dynamic streams
nSt=readxl(f, "Streamdata!I2:I2"); nSt=Int(nSt[1]);         #no. of static streams

#remember to update cellnumbers
TS=readxl(f, "Streamdata!C5:C11");                          #supply temp 
TT=readxl(f, "Streamdata!D5:D11");                          #target temp 
Pr=readxl(f, "Streamdata!G5:G8");                           #pressure ratio
mCp=readxl(f, "Streamdata!H5:H11");                         #flow rate capacity 

#remember to update values for below ambient cases
DTmin=readxl(f, "Streamdata!A2:A2"); DTmin=Int(DTmin[1]);   #delta T min
Tamb=readxl(f, "Streamdata!B2:B2");  Tamb=Float64(Tamb[1]); #ambient temp
Thu=readxl(f, "Streamdata!C2:C2");   Thu=Float64(Thu[1]);   #hot utility temp
Tcu=readxl(f, "Streamdata!D2:D2");   Tcu=Float64(Tcu[1]);   #cold utility temp
κ=readxl(f, "Streamdata!E2:E2");     κ=Float64(κ[1]);       #kappa
η=readxl(f, "Streamdata!F2:F2");     η=Float64(η[1]);       #polytropic efficiency 
n=readxl(f, "Streamdata!H2:H2");     n=Int(n[1]);           #number of stream splits
T=273.15;                                                   #kelvin temperature




#decoding of genes from genotype to phenotype representation

function genotypeMap(genPop, paraNum, ub, lb) 

popSize = length(genPop[:,1]);      #population size equals number of chromosomes
phenPop = zeros(popSize, paraNum);  #empty population in phenotype representation 
stepSize = zeros(Float64, paraNum); #discretization of variable range

#calculate the resolution for each gene
for i=1:paraNum
    stepSize[i] = (ub-lb)/((2^nBits)-1); #evenly distr density of variable range
end 

#for all individuals calculate the phenotype representation of each gene
for i = 1:popSize                    
    k = 1;                           
    for j = 1:paraNum
        phenPop[i,j] = (parse(Int64, genPop[i][k:k+nBits-1], 2))*stepSize[j]+lb   
        k = k+nBits
    end 
end 
return phenPop                       #return a population of phenotype values
end

#fitness function/the innter loop

function computeFitness(phenPop)

#output values for comparison of solutions
    
exergy = zeros(length(phenPop[:,1]));       #fitness value of individuals in pop p
Uhot = zeros(length(phenPop[:,1]));         #hot utility of indiv solutions in pop p
Ucold = zeros(length(phenPop[:,1]));        #cold utility of indiv solutions in pop p    
mfr = zeros(nPr, n, length(phenPop[:,1]));  #mCp of each stream split in pop p
TIN = zeros(nPr, n, length(phenPop[:,1]));  #input temp to pressure changing units
TOUT = zeros(nPr, n, length(phenPop[:,1])); #output temp to pressure changing units
Work = zeros(nPr, n, length(phenPop[:,1])); #work from each pressure changing unit

    #------------------------start preprocessing step------------------------

    #for each individual solution in population p we calculate the fitness value
    for i=1:length(phenPop[:,1])                                
        
        #extrac info from genes and calculate pressure ratio and output temperatures
        Tin = collect(reshape(phenPop[i,:], nPr, n)); 
        
        PR = []; Tout = zeros(nPr, n);
       
        PR = collect((Pr[s])^((κ-1)/κ) for s=1:length(Pr));                       
        
        for s = 1:nPr
            for b = 1:n
                Tout[s,b] = ((Tin[s,b]+273.15)/(PR[s]))-273.15; 
            end 
        end 
                
                
        #[split operator] 
        #generate new streams; one segment before unit, one after unit
        #[OBS!] pressure changing streams must be listed first in stream data
                
        TSnew = zeros(2*nPr*n); 
        TTnew = zeros(2*nPr*n); 
        mCpnew = zeros(2*nPr*n); 
        W = zeros(nPr, n);
                
        k = 1;
        for s = 1:nPr
            for b = 1:n
                TSnew[k] = TS[s];             #supply temp of segment before unit 
                TSnew[k+1] = Tout[s,b];       #supply temp of segment after unit
                TTnew[k] = Tin[s,b];          #target temp of segment before unit    
                TTnew[k+1] = TT[s];           #target temp of segment after unit
                mCpnew[k] = mCp[s];           #mCp of segment before unit
                mCpnew[k+1] = mCp[s];         #mCp of segment after unit
                k = k+2; 
                    
                W[s,b] = (Tout[s,b]-Tin[s,b])*mCp[s]; #work in each unit
            end 
        end 
       
        #for later simplicity we add the static streams to the new vectors
                
        append!(TSnew, TS[nPr+1:end]);        
        append!(TTnew, TT[nPr+1:end]);        
        append!(mCpnew, mCp[nPr+1:end]);      
                                 
        #generate the heat cascade: 
        #1. the cold temperature side 
        #2. the hot temperature side 
        #3. number of temperature intervals
                
        Tcold = []; Thot = []; K = [];        
        for j = 1:length(TSnew)               #create heat casc from all supply temp
            if TSnew[j] >= TTnew[j]           #hot stream
               push!(Tcold, TSnew[j]-DTmin);  #adjust for delta Tmin
            else push!(Tcold, TSnew[j])       #cold stream
            end
        end
        
        #to create the heat cascade we need to add the max/min temps in the system
        push!(Tcold, min(Tamb,Tcu));                                   
                                    
        Tcold = reverse(sort(union(Tcold)));  #sort the cold temp side
        Thot = Tcold+DTmin;                   #create hot side
        K = length(Tcold);                    #no. of temp intervals
        
        #for testing/debugging of function:        
        #println("Tin:  ", Tin)
        #println("Tout:  ", Tout)
        #println("TSnew:  ", TSnew)
        #println("TTnew:  ", TTnew)
        #println("mCpnew:  ", mCpnew)
        #println("Work:  ", W)
        #println("Tcold:  ", Tcold)
        #println("Thot:  ", Thot)
        #println("K:  ", K)
        
        #for simplicity, the heat in each interval from the static streams..
        #..and the pressure changing streams are calculated separately.
        
        #STATAIC STREAMS
        #heat from hot and cold streams to interval k in K
                
        QS = zeros(nSt, K); q = 1;
                            
for s = nPr+1:length(TS)
    if TS[s] > TT[s]
                    
    for k = 1:K-1  
        if (TT[s]<=Thot[k+1]||(TT[s]<=Thot[k]&&TT[s]>=Thot[k+1]))&&TS[s]>=Thot[k]
            QS[q,k] = mCp[s]*(Thot[k]- max(Thot[k+1], TT[s]))
        end
    end
                
                    
    elseif TS[s] < TT[s]
                  
    for k = 1:K-1
        if TS[s]<=Tcold[k+1]&&(TT[s]>=Tcold[k]||(TT[s]<=Tcold[k]&&TT[s]>=Tcold[k+1]))
            QS[q,k] = mCp[s]*(Tcold[k+1] - min(Tcold[k], TT[s]))
        end
    end
                   
end
q = q+1;
end   
               
        #for simplicity, sum heat in each interval
                
        Qksum = zeros(K)
        for k = 1:K
            Qksum[k] = sum(QS[1:end,k])
        end
        
                
        #PRESSURE CHANGING STREAMS
        #heat from pressure changing streams depends on the variable mass fraction
        #we calculate the heat supply/demand beforehand - 
        #and multiply with a fraction variable 0<=f<=1 in the optimization model
                
        #heat from hot and cold streams to interval k in K
                
        QP = zeros(2*nPr*n, K); p = 1;
                            
for s = 1:2*nPr*n
            
if TSnew[s] > TTnew[s]
    
for k = 1:K-1  
if (TTnew[s]<=Thot[k+1]||
    (TTnew[s]<=Thot[k]&&TTnew[s]>=Thot[k+1]))&&TSnew[s]>=Thot[k]
     QP[p,k] = mCpnew[s]*(Thot[k]- max(Thot[k+1], TTnew[s]));
end
end
                
            
elseif TSnew[s] < TTnew[s]
                 
for k = 1:K-1
if TSnew[s]<=Tcold[k+1]&&(TTnew[s]>=Tcold[k]||
    (TTnew[s]<=Tcold[k]&&TTnew[s]>=Tcold[k+1]))
    QP[p,k] = mCpnew[s]*(Tcold[k+1] - min(Tcold[k], TTnew[s]));
end
end
                   
end
p = p+1;
end 
        
        #heat content of pressure changing stream segments are calculated above
        #these are listed in the vector QP[p,k] 
        #p is the no. of stream segments and k is the no. of temp intervals 
        #segments belonging to the same branch has the same flow fraction, f
        #hence, the array is reshaped to the form QP[z,n,nPr,k] 
        #for each interval k -
        #sum for all pressure changing streams and all branches
        #ex. sum(QP[z,1,1,1]*f[1,1] for z = {1,2}, s = 1, b = 1, k = 1)
                
        #reshape QP so to be compatible with f[s,b]
                
        Z = 2;
        QPr = reshape(QP, Z, n, nPr, K);  
                
        #-------------------------end preprocessing step-------------------------
             
        #LP-model  
                
        HeatCasc=Model(solver=CplexSolver())
                
        @variable(HeatCasc, r[1:K] >= 0);
        @variable(HeatCasc, Qhu >= 0);
        @variable(HeatCasc, 0 <= f[s = 1:nPr, b = 1:n] <= 1);
                                                                  
        @objective(HeatCasc, Min, 
                    (1-(Tamb+T)/(Thu+T))*Qhu + ((Tamb+T)/(Tcu+T)-1)*r[K-1] + 
                    sum(f[s,b]*W[s,b] for s=1:nPr, b=1:n));
                             

        @constraint(HeatCasc, FirstInt[k = 1], 
                    r[k] - Qhu == 
                    sum(QPr[z,b,s,k]*f[s,b] for s=1:nPr, b=1:n, z=1:Z) + Qksum[k]);
                                        
        @constraint(HeatCasc, ResInt[k = 2:K], 
                    r[k] - r[k-1] == 
                    sum(QPr[z,b,s,k]*f[s,b] for s=1:nPr, b=1:n, z=1:Z) + Qksum[k] );
                                                    
        @constraint(HeatCasc, frac[s = 1:nPr], sum(f[s,b] for b = 1:n) == 1);
        
        #avoid print put from the optimization                                                        
        WW = STDOUT;                                        
        redirect_stdout();
        solve(HeatCasc);                                                                        
        redirect_stdout(WW);                                       
       
        exergy[i] = getobjectivevalue(HeatCasc);     
        Uhot[i] = getvalue(Qhu); 
        Ucold[i] = getvalue(r[K-1]);
        for s = 1:nPr
            for  b = 1:n
                 mfr[s,b,i] = getvalue(f[s,b]);      
                 Work[s, b, i] = W[s, b];            
                 TIN[s, b, i] = Tin[s, b];           
                 TOUT[s, b, i] = Tout[s, b];                                                   
            end
        end
        
        #for testing/debugging of function:                                                       
        #println("exergy:   ",exergy)
        #println("Qhu:      ",Uhot)
        #println("mfr:      ",mfr)
        #println("Work:     ",Work)
        #println("Tin:      ",TIN)
        #println("Tout:     ",TOUT)
                                                            
    end
                                                            
        indEx = indmin(exergy);
        bestfit_Uhot = Uhot[indEx];
        bestfit_work = collect(Work[s,b,i] for s = 1:nPr, b = 1:n, i = indEx);
        bestfit_TIN = collect(TIN[s,b,i] for s = 1:nPr, b = 1:n, i = indEx);  
        bestfit_TOUT = collect(TOUT[s,b,i] for s = 1:nPr, b = 1:n, i = indEx);                                                                          
        bestfit_mfr = collect(mfr[s,b,i] for s = 1:nPr, b = 1:n, i = indEx); 
                                                                                                                        
    return exergy, Uhot, Work, TIN, TOUT, mfr; #return solution and fitness measure
end                                                           
                                                                                                    
                                                                                                    

#elitist selection
#the very best individuals directly survive to the next generation 

function elitistSel(fitness, ε)
    
    popSize = length(fitness[:,1]);    #population size
    M = maximum(fitness) + 100;        #avoid choosing the same individual
    nε = Int(round(popSize*ε));        #number of "free survivors" 
    fitVal = collect(fitness);                  
    
    E = zeros(Int, nε)
    for i=1:nε
        bestfit=indmin(fitVal);        #returns a tuple of (val,indx)
        E[i] = bestfit;                #save the position of the best individual
        fitVal[bestfit] = M;           #remove best value to find second best one
    end
    return E
end

#how easily find the index of the n maximum values of a vector?!
#should be a function for this in julia, but cannot find any

#tournament selection

function tournamentSel(tourSize, fitness, ε)

  popSize = length(fitness[:,1]);  
  indxPool = [];                        #indexes of winners from the tournament
  nε = Int(round(popSize*ε));           #number of "free survivors"
    
  for i = 1:(popSize-nε)                #number of parents, λ, to be selected
    indx = rand(1:popSize, tourSize);   #randomly choose a sub group of the pop
    tourfit = fitness[indx];            #assign fitness measure
    winner = indmin(tourfit);           #choose the fittest
    push!(indxPool, indx[winner]);      #the indiv with highest fitness selected
  end                                    
  return indxPool                       #return the mating pool of indx values
    
end

#single point crossover operator

function crossSinglePoint(selPop, α) 

popSize = length(selPop);                #population size
selVec = collect(1:popSize);             #selection vector
crossPop = Array{String}(popSize);       #population of offsprings

i = 1
while i <= popSize-1
    
      indxA = rand(1:length(selVec));    #random parent chosen from mating pool
      parentA = selPop[selVec[indxA]];   #parent subject to crossover only once
      deleteat!(selVec, indxA);
    
     if rand() <= α                      #probability of crossover
        indxB = rand(1:length(selVec));  
        parentB = selPop[selVec[indxB]]; #second parent chosen from mating pool
        deleteat!(selVec, indxB);        #parent subject to crossover only once
        
            #location of pointer
            pointer = rand(1:length(parentA)); 

            #generate new individuals from parents
            childA = parentA[1:pointer] * parentB[pointer+1:end];   
            childB = parentB[1:pointer] * parentA[pointer+1:end];   
        
            #place new individuals in new population
            crossPop[i] = childA                                    
            crossPop[i+1] = childB                                   
            i = i+2;
     else crossPop[i] = parentA;         #parent survive
          i = i+1;
    end
end

if isempty(selVec) == false
   crossPop[popSize] = selPop[selVec[1]];
end
    
return crossPop                          #return new population
end

#uniform crossover operator

function uniformCrossover(selPop, α) 


popSize = length(selPop);                #population size
selVec = collect(1:popSize);             #selection vector
crossPop = Array{String}(popSize);       #population of offsprings

i = 1
while i <= popSize-1
    
      indxA = rand(1:length(selVec));    #random parent chosen from mating pool
      parentA = selPop[selVec[indxA]];   #parent subject to crossover only once
      deleteat!(selVec, indxA);
    
     if rand() <= α                      #probability of crossover
        indxB = rand(1:length(selVec));
        parentB = selPop[selVec[indxB]]; #second parent chosen from mating pool     
        deleteat!(selVec, indxB);        #parent subject to crossover only once
            
            #uniform crossover mask
            mask = join(rand(0:1, nBits*nPr*n));
            
            #generate new individuals from parents
            newbornA = zeros(Int, length(mask));
            newbornB = zeros(Int, length(mask));
            
            for a = 1:length(mask)
                if parse(Int, mask[a]) == 1
                    newbornA[a] = parse(Int, parentA[a]);
                    newbornB[a] = parse(Int, parentB[a]);
                    
                elseif parse(Int, mask[a]) == 0
                    newbornA[a] = parse(Int, parentB[a]);
                    newbornB[a] = parse(Int, parentA[a]);
                end
            end
            
            childA = join(newbornA);
            childB = join(newbornB);
        
            #place new individuals in new population    
            crossPop[i] = childA              
            crossPop[i+1] = childB            
            i = i+2;
     else crossPop[i] = parentA;         #parent survive
          i = i+1;
    end
end

if isempty(selVec) == false
   crossPop[popSize] = selPop[selVec[1]];
end
    
return crossPop                          #return new population
end



#single bitswap mutation operator

function mutationSingleBitswap(crossPop, μ)

popSize = length(crossPop);      #population size
chromSize = length(crossPop[1]); #chromosome size
    
for i = 1:popSize
        
    if rand() <= μ               #mutation probability

       flip = rand(1:chromSize); #flip position
            
       newGen = zeros(Int, chromSize);
       for j = 1:chromSize
           if j == flip
              if parse(Int, crossPop[i][j]) == 0
                 newGen[j] = 1;
              elseif parse(Int, crossPop[i][j]) == 1 
                 newGen[j] = 0;
              end
           else
                newGen[j] = parse(Int, crossPop[i][j]);
           end     
        end
            
        crossPop[i] = join(newGen);
        
    #else println("no mutation")
    end 
end

return crossPop                  #return mutated population
end


#flipping bitswap mutation operator

function mutationFlippingBitswap(crossPop, μ)

popSize = length(crossPop);             #population size
chromSize = length(crossPop[1]);        #chromosome size  
    
for i = 1:popSize  
    
    if rand() <= μ                      #mutation probability

       mutChrom = rand(0:1, chromSize); #mutation chromosome   
       newGen = zeros(Int, chromSize);
       for j = 1:chromSize
            
           if mutChrom[j] == 1
                
              if parse(Int, crossPop[i][j]) == 0
                 newGen[j] = 1;
              elseif  parse(Int, crossPop[i][j]) == 1
                  newGen[j] = 0;
              end
           else
                newGen[j] = parse(Int, crossPop[i][j]);
           end     
        end
           
        crossPop[i] = join(newGen);
       
    #else println("no mutation")
    end 
end

return crossPop                         #return mutated population
end

#transition to next generation

function nextGeneration(crossPop, Eindx, genPop)
    
    crossSize = length(crossPop);              #population size
    newGenPop = Array{String}(length(genPop)); #new population
  
    for i = 1:crossSize
        newGenPop[i] = crossPop[i]; 
    end 
   
    k = 1
    for j = crossSize+1:popSize
        newGenPop[j] = genPop[Eindx[k]];
        k = k+1
    end
    
return newGenPop #return new population in binary representation    
end

#BGA MAIN FILE

tic()                  #start timing the computation

# ------------initialization of the GA-----------------------------------------
paraNum = nPr*n;       #no. of decision variables featuring one solution
lb = Tamb;             #lower bound on range for decision variables in paraNum
ub = Thu;              #upper bound on range for decision variables in paraNum
nBits = 11;            #number of bits needed to represent the variable range 
                       #in binary numbers
ε = 0.01;              #rate of elitism
α = 0.75;              #crossover probability
μ = 0.10;              #mutation rate 1/(nBits*nPr*n)
tourSize = 2;          #tournament size
popSize = 20*paraNum;  #size of a population
genCount = 10000;      #generation counter
#----------------------------------------------------------------------------

#initial population in chromosome representation

#genPop -> [Tin,sb] -> [Tin,11 Tin,21 Tin,12 Tin,22 Tin,13 Tin,23 for s=2, n=3]
genPop = Array{String}(popSize)            
seed = 123;
for i = 1:popSize
    srand(seed);
    genPop[i] = join(rand(0:1, nBits*nPr*n));
    seed = seed + i;
end

bestfit = [];
averagefit = [];

#------------------------------start outer loop----------------------------------

for m = 1:genCount
    
    #mapping of genotype to phenotype
    phenPop = genotypeMap(genPop, paraNum, ub, lb);                              
    
    #--------------------------inner loop------------------------------------
    
    #compute fitness value of the individuals
    fitness = computeFitness(phenPop);
    
    #------------------------------------------------------------------------
    
    #best solution found for this population 
    push!(bestfit, minimum(fitness[1])); 
    push!(averagefit, sum(fitness[1]/length(fitness[1])));

    #selection
    #directly winners to next generation 
    Eindx = elitistSel(fitness[1], ε); 
    
    #mating pool with higher average fitness values
    matPool = tournamentSel(tourSize, fitness[1], ε);
    
    #binary representation of selected individuals 
    selPop = genPop[matPool];                                      

    #crossover
    crossPop = uniformCrossover(selPop, α);            

    #mutation
    crossPop = mutationFlippingBitswap(crossPop, μ);                 
    
    #transition to next generation
    genPop = nextGeneration(crossPop, Eindx, genPop);         
    
    #for testing/debugging of function:
    #println("fitness:    ",fitness)
    #println("winners:    ",winners)
    #println("selPop:     ",selPop)
    #println("Elitism:    ",E)
    #println("crossPop:   ", crossPop)
    #println("genPop:     ",genPop)
    #println("iteration:  ", m)
    
    if m == genCount  
    
    #for testing/debugging of function:
    #println("exergy:  ", fitness[1])
    #println("hot utility:  ", fitness[2])
    #println("mass flow fraction:  ", fitness[6])
    #println("work:  ", fitness[3])
    #println("Tin:  ", fitness[4])
    #println("Tout:  ", fitness[5])
    
    
    bf = minimum(fitness[1]);
    indVec = [];
    for i = 1:length(fitness[1])
        if fitness[1][i] == bf;
           push!(indVec, i);
        end                                                                                            
    end
        
    numSol = length(indVec);
    println("                                                                  ")                                                                                            
    println("Results from SGA last iteration m = ",m)
    println("                                                                  ")                                                                                                    
    println("number of optimal solutions found:  ",numSol)                                                                     
    println("__________________________________________________________________")
    println("                                                                  ") 
    for i = 1:length(indVec)                                                                                                                                                                                         
        println("Solution $i:                                                  ") 
        println("                                                              ")                                                                                                                                                                                                       
        println("Exergy value:                 ", fitness[1][indVec[i]])                                                                                                  
        println("Hot utility:                  ", fitness[2][indVec[i]]) 
        println("Work:                         ", fitness[3][:,:,indVec[i]])                                                                                                    
        println("Fraction of mass flow rate:   ", fitness[6][:,:,indVec[i]])                                                                                                    
        println("Input temperature to unit:    ", fitness[4][:,:,indVec[i]])        
        println("Output temperature from unit: ", fitness[5][:,:,indVec[i]])                                                                                                    
        println("______________________________________________________________")                                                                                                    
        println("                                                              ")                                                                                                    
        println("                                                              ")     
    end 
 end
      
end

#------------------------------end outer loop------------------------------------

toc() #end timing

#plot average population fitness and best fitness value throughout the search

using Gadfly   #for plotting
plot(x=1:genCount, y=bestfit, Geom.line)

using DataFrames

xs=1:genCount;

df_bestfit=DataFrame(x=xs, y=bestfit, 
    Properties="Highest fitness in population x")
df_averagefit=DataFrame(x=xs,y=averagefit,
    Properties="Average fitness in population x")
df=vcat(df_bestfit, df_averagefit)
p=plot(df, x=:x, y=:y, color=:Properties, Geom.line, 
      Guide.title("Population fitness"),
      Guide.xlabel("Population iterations"), Guide.ylabel("Fitness values") )

draw(SVGJS(25cm, 16cm), p)



